# SPDX-FileCopyrightText: 2025-present Marceau <git@marceau-h.fr>
#
# SPDX-License-Identifier: AGPL-3.0-or-later
from enum import Enum

class Formats(Enum):
    video = {".mp4", ".mkv", ".avi", ".mov", ".wmv", ".flv", ".webm", ".m4v", ".mpeg", ".mpg", ".3gp", ".3g2", ".vob", ".ts", ".m2ts", ".rmvb", ".mxf", ".drc", ".amv", ".f4v", ".svi", ".m1v", ".m2v"}
    image = {".jpg", ".jpeg", ".png", ".gif", ".bmp", ".tiff", ".webp", ".svg", ".heif", ".raw"}
    audio = {".mp3", ".wav", ".flac", ".aac", ".ogg", ".wma", ".m4a", ".opus", ".alac", ".aiff"}
    plain_text = {".txt"}
    csv = {".csv"}
    json = {".json"}
    html = {".html", ".htm"}
    xml = {".xml"}
    excel = {".xls", ".xlsx", ".odf", ".ods"}
    pdf = {".pdf"}
    doc = {".doc", ".docx", ".odt"}
    ppt = {".ppt", ".pptx", ".odp"}
    archive = {".zip", ".tar", ".gz", ".bz2", ".xz", ".7z", ".rar", ".tar.gz", ".tar.bz2", ".tar.xz"}
    text = {".txt", ".csv", ".json", ".xml", ".html", ".htm", ".py", ".java", ".c", ".cpp", ".h", ".js", ".css", ".md", ".rst", ".tex", ".sql", ".css", ".yaml", ".yml", ".toml", ".ini", ".properties", ".log"}

    def __hash__(self) -> int:
        return hash(self.value)

    def __eq__(self, other: "Formats") -> bool:
        if isinstance(other, Formats):
            return self.value == other.value
        return False

    def __ne__(self, other: "Formats") -> bool:
        if isinstance(other, Formats):
            return self.value != other.value
        return True

    def __contains__(self, other: str) -> bool:
        if isinstance(other, str):
            return other in self.value
        return False



# SPDX-FileCopyrightText: 2025-present Marceau <git@marceau-h.fr>
#
# SPDX-License-Identifier: AGPL-3.0-or-later
from pathlib import Path
from typing import Generator, Optional, Set, Union

from .Formats import Formats

class FileDetect:
    def __init__(
            self,
            /,
            *args,
            format: Optional[Union[str, Formats]] = None,
            deep: int = -1,
            only_stems: Optional[Set[str]] = None,
            suffixes: Optional[Union[Set[str], str]] = None,
            **kwargs
    ):
        if isinstance(deep, str):
            try:
                deep = int(deep)
            except ValueError:
                raise ValueError(f"ERROR : invalid deepness treshold : {deep}")
        elif not isinstance(deep, int) or deep < -1:
            raise ValueError(f"ERROR : invalid deepness treshold : {deep}")

        if isinstance(format, Formats):
            suffixes = format.value
        elif isinstance(format, str):
            try:
                format = Formats[format]
                suffixes = format.value
            except KeyError:
                raise ValueError(f"ERROR : invalid file type : {format}")
        elif format is None:
            if isinstance(suffixes, str):
                suffixes = {suffixes}
            elif isinstance(suffixes, set):
                pass
            elif suffixes is None:
                pass
            else:
                raise ValueError(f"ERROR : invalid suffixes : {suffixes}")
        else:
            raise ValueError(f"ERROR : invalid file type : {format}")


        self.format = format
        self.deep = deep
        self.only_stems = only_stems
        self.suffixes = suffixes
        self.result: Set[Path] = set()

    def run(self, path: Union[str, Path], *args, **kwargs) -> Set[Path]:
        self.result |= set(self(path))
        return self.result

    def __call__(self, path: Union[str, Path], *args, **kwargs) -> Generator[Path, None, None]:
        if self.deep != -1 and self.deep < 0:
            self.deep -= 1

        if isinstance(path, str):
            path = Path(path)
        elif not isinstance(path, Path):
            raise ValueError(f"ERROR : invalid path : {path}")

        for file in path.glob("*"):
            if file.is_dir():
                if self.deep != 0:
                    yield from self(file)
            elif not self.suffixes or file.suffix in self.suffixes:
                if self.only_stems is not None:
                    if file.stem in self.only_stems or any(file.stem.startswith(stem) for stem in self.only_stems):
                        yield file
                else:
                    yield file


    def __len__(self):
        return len(self.result)

    def __iter__(self):
        return iter(self.result)

    @classmethod
    def find(
            cls,
            path: Union[str, Path],
            *args,
            deep: int = -1,
            format: Optional[Union[str, Formats]] = None,
            only_stems: Optional[Set[str]] = None,
            suffixes: Optional[Union[Set[str], str]] = None,
            **kwargs
    ) -> "FileDetect":
        finder = cls(
            path,
            deep=deep,
            format=format,
            only_stems=only_stems,
            suffixes=suffixes,
            **kwargs
        )
        finder.run(path, *args, **kwargs)
        return finder
